#include <cctype> 
#include <sstream>
#include <string>
#include "node.hpp"
#include "parser.hpp"

const char *names[] = {
			"NUM", "X", "PLUS", "MINUS", "MUL", "DIV", "POW", 
			"LPAR", "RPAR", "LOG", "EXP", "SIN", "COS", "ATAN", "SQRT",
			"DER", "LFT", "RGT", "SEMI", "BAD"
};

Parser::Parser(string str, Node* u, Node *v) : istringstream(str)
{
	lft= u;
	rgt= v;
}

void Parser::error(string msg)
{
	cerr << "ERROR: " << msg << end;
	exit(1);
}

int Parser::gettok(void)
{
	int  ch;
	*this >> ws;			// ws = white space 
	ch= this->get();
	if (isalpha(ch)) {
		string s="";
		do {
			s+= char(ch);
			ch= this->get();
		} while (isalpha(ch));
		this->putback(ch);

		if (s=="x")
			tok= X;
		else
		if (s=="log")
			tok= LOG;
		else
		if (s=="exp")
			tok= EXP;
		else
		if (s=="sin")
			tok= SIN;
		else
		if (s=="cos")
			tok= COS;
		else
		if (s=="atan")
			tok= ATAN;
		else
		if (s=="sqrt")
			tok= SQRT;
		else
		if (s=="der")
			tok= DER;
		else
		if (s=="lft")
			tok= LFT;
		else
		if (s=="rgt")
			tok= RGT;
		else
			tok= BAD;
	} 
	else
	switch(ch) {
		case '0': case '1': case '2':
		case '3': case '4': case '5':
		case '6': case '7': case '8':
		case '9':	
					this->putback(ch);
					*this >> num;
					tok= NUM;
					break;
		case 'x':
		case 'X':	tok= X;
					break;
		case '+': 	tok= PLUS;
					break;
		case '-': 	tok= MINUS;
					break;
		case '*':	tok= MUL;
					break;
		case '/':	tok= DIV;
					break;
		case '^':	tok= POW;
					break;
		case '(':	tok= LPAR;
					break;
		case ')':	tok= RPAR;
					break;
		case ';':	tok= SEMI;
					break;
		default:	tok= BAD;
					break;
	}
//	cerr << "DEBUB: " << names[tok] << endl;
	return tok;
}

Node* Parser::prim(void)
{
	Node *t;
	switch(tok) {
		case NUM:
				t= new Const(num);
				gettok();
				break;

		case X: 
				t= new Var();
				gettok();
				break;

		case LFT:
				t= lft->copy();
				gettok();
				break;
			
		case RGT:
				t= rgt->copy();
				gettok();
				break;

		case LPAR:
				gettok();
				t= expr();
				if (tok==RPAR)
					gettok();
				else
					error("missing ')'");
				break;

		case EXP:
				gettok();
				if (tok==LPAR) {
					gettok();
					t= new Exp(expr());
					if (tok==RPAR)
						gettok();
					else
						error("missing ')'");
				}
				else
					error("missing (expr)");
				break;

		case LOG:
				gettok();
				if (tok==LPAR) {
					gettok();
					t= new Log(expr());
					if (tok==RPAR)
						gettok();
					else
						error("missing ')'");
				}
				else
					error("missing (expr)");
				break;

		case DER:
				gettok();
				if (tok==LPAR) {
					gettok();
					t= new Der(expr());
					if (tok==RPAR)
						gettok();
					else
						error("missing ')'");
				}
				else
					error("missing (expr)");
				break;
	}
	return t;
}

Node* Parser::factor(void)
{
	Node* t= prim();

	while (tok==POW) {
		gettok();
		t= new Pow(t, factor());
	}	
	return t;
}

Node* Parser::term(void)
{
	Node* t= factor();

	while (tok==MUL || tok==DIV) {
		if (tok==MUL) {
			gettok();
			t= new Mul(t, factor());
		}
		else {
			gettok();
			t= new Div(t, factor());
		}
	}
	return t;
}

Node* Parser::expr(void)
{
	Node* t= term();

	while (tok==PLUS || tok==MINUS) {
		if (tok==PLUS) {
			gettok();
			t= new Add(t, term());
		}
		else {
			gettok();
			t= new Sub(t, term());
		}
	}
	return t;
}

Node* Parser::line(void)
{
	gettok();
	Node *t= expr();
	if (tok==SEMI)
		return t;
	else
		return NULL;
}

